@@ -64,31 +64,32 @@
 #' \code{\link[evmix:fbckdengpd]{fbckdengpd}} returns a simple list with the following elements
 #'
 #' \tabular{ll}{
-#'  \code{call}:    \tab \code{optim} call\cr
-#'  \code{x}:       \tab data vector \code{x}\cr
-#'  \code{init}:    \tab \code{pvector}\cr
-#'  \code{fixedu}:  \tab fixed threshold, logical\cr
-#'  \code{useq}:    \tab threshold vector for profile likelihood or scalar for fixed threshold\cr
-#'  \code{optim}:   \tab complete \code{optim} output\cr
-#'  \code{mle}:     \tab vector of MLE of parameters\cr
-#'  \code{cov}:     \tab variance-covariance matrix of MLE of parameters\cr
-#'  \code{se}:      \tab vector of standard errors of MLE of parameters\cr
-#'  \code{rate}:    \tab \code{phiu} to be consistent with \code{\link[evd:fpot]{evd}}\cr
-#'  \code{nllh}:    \tab minimum negative log-likelihood\cr
-#'  \code{n}:       \tab total sample size\cr
-#'  \code{lambda}:  \tab MLE of lambda (kernel half-width)\cr
-#'  \code{u}:       \tab threshold (fixed or MLE)\cr
-#'  \code{sigmau}:  \tab MLE of GPD scale\cr
-#'  \code{xi}:      \tab MLE of GPD shape\cr
-#'  \code{phiu}:    \tab MLE of tail fraction (bulk model or parameterised approach)\cr
-#'  \code{se.phiu}: \tab standard error of MLE of tail fraction\cr
-#'  \code{bw}:      \tab MLE of bw (kernel standard deviations)\cr
-#'  \code{kernel}:  \tab kernel name\cr
-#'  \code{bcmethod}:\tab boundary correction method\cr
-#'  \code{proper}:  \tab logical, whether renormalisation is requested\cr
-#'  \code{nn}:      \tab non-negative correction method\cr
-#'  \code{offset}:  \tab offset for log transformation method\cr
-#'  \code{xmax}:    \tab maximum value of scaled beta or copula
+#'  \code{call}:      \tab \code{optim} call\cr
+#'  \code{x}:         \tab data vector \code{x}\cr
+#'  \code{init}:      \tab \code{pvector}\cr
+#'  \code{fixedu}:    \tab fixed threshold, logical\cr
+#'  \code{useq}:      \tab threshold vector for profile likelihood or scalar for fixed threshold\cr
+#'  \code{nllhuseq}:  \tab profile negative log-likelihood at each threshold in useq\cr
+#'  \code{optim}:     \tab complete \code{optim} output\cr
+#'  \code{mle}:       \tab vector of MLE of parameters\cr
+#'  \code{cov}:       \tab variance-covariance matrix of MLE of parameters\cr
+#'  \code{se}:        \tab vector of standard errors of MLE of parameters\cr
+#'  \code{rate}:      \tab \code{phiu} to be consistent with \code{\link[evd:fpot]{evd}}\cr
+#'  \code{nllh}:      \tab minimum negative log-likelihood\cr
+#'  \code{n}:         \tab total sample size\cr
+#'  \code{lambda}:    \tab MLE of lambda (kernel half-width)\cr
+#'  \code{u}:         \tab threshold (fixed or MLE)\cr
+#'  \code{sigmau}:    \tab MLE of GPD scale\cr
+#'  \code{xi}:        \tab MLE of GPD shape\cr
+#'  \code{phiu}:      \tab MLE of tail fraction (bulk model or parameterised approach)\cr
+#'  \code{se.phiu}:   \tab standard error of MLE of tail fraction\cr
+#'  \code{bw}:        \tab MLE of bw (kernel standard deviations)\cr
+#'  \code{kernel}:    \tab kernel name\cr
+#'  \code{bcmethod}:  \tab boundary correction method\cr
+#'  \code{proper}:    \tab logical, whether renormalisation is requested\cr
+#'  \code{nn}:        \tab non-negative correction method\cr
+#'  \code{offset}:    \tab offset for log transformation method\cr
+#'  \code{xmax}:      \tab maximum value of scaled beta or copula
 #' }
 #' 
 #' @note 
@@ -138,8 +139,8 @@
 #' @author Yang Hu and Carl Scarrott \email{carl.scarrott@@canterbury.ac.nz}
 #'
 #' @section Acknowledgments: See Acknowledgments in
-#'   \code{\link[evmix:fnormgpd]{fnormgpd}}, type \code{help fnormgpd}. Based on MATLAB
-#'   code written by Anna MacDonald.
+#'   \code{\link[evmix:fnormgpd]{fnormgpd}}, type \code{help fnormgpd}. Based on code
+#' by Anna MacDonald produced for MATLAB.
 #' 
 #' @seealso \code{\link[evmix:kernels]{kernels}}, \code{\link[evmix:kfun]{kfun}},
 #'  \code{\link[stats:density]{density}}, \code{\link[stats:bandwidth]{bw.nrd0}}
@@ -151,7 +152,9 @@
 #' 
 #' @examples
 #' \dontrun{
-#' par(mfrow=c(2,1))
+#' set.seed(1)
+#' par(mfrow = c(2, 1))
+#' 
 #' x = rgamma(500, 2, 1)
 #' xx = seq(-0.1, 10, 0.01)
 #' y = dgamma(xx, 2, 1)
@@ -198,26 +201,27 @@ fbckdengpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NU
 
   call <- match.call()
     
+  np = 4 # maximum number of parameters
+
   # Check properties of inputs
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
-  check.logic(logicarg = phiu) # only logical
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
+  check.logic(phiu)
   check.posparam(useq, allowvec = TRUE, allownull = TRUE)
-  check.logic(logicarg = fixedu)
-  check.logic(logicarg = std.err)
+  check.logic(fixedu)
+  check.logic(std.err)
   check.optim(method)
   check.control(control)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
 
   check.kernel(kernel)
   check.bcmethod(bcmethod)
-  check.logic(logicarg = proper)
+  check.logic(proper)
   check.nn(nn)
-  check.posparam(offset, allownull = TRUE)  
-  check.offset(offset, bcmethod)
+  check.offset(offset, bcmethod, allowzero = TRUE)
   check.posparam(xmax, allownull = TRUE)  
-  check.posparam(param = factor)
-  check.posparam(param = amount, allownull = TRUE)
-  check.logic(logicarg = add.jitter)
+  check.posparam(factor)
+  check.posparam(amount, allownull = TRUE)
+  check.logic(add.jitter)
   
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -231,7 +235,6 @@ fbckdengpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NU
   
   check.quant(x)
   n = length(x)
-  np = 4 # maximum number of parameters
 
   if (add.jitter) x = pmax(jitter(x, factor, amount), 0)
 
@@ -271,6 +274,7 @@ fbckdengpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NU
     }
     # need to recheck there are some valid kernel centres after these exclusions
     check.quant(x)
+    n = length(x)
 
     if (max(x) > xmax) stop("largest kernel centre must be below xmax")   
   }
@@ -333,7 +337,7 @@ fbckdengpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NU
       
       # remove thresholds with less than 5 excesses
       useq = useq[sapply(useq, FUN = function(u, x) sum(x > u) > 5, x = x)]
-      check.param(useq, allowvec = TRUE)
+      check.posparam(useq, allowvec = TRUE)
       
       nllhu = sapply(useq, proflubckdengpd, pvector = pvector, x = x, phiu = phiu, kernel = kernel,
         bcmethod = bcmethod, proper = proper, nn = nn, offset = offset, xmax = xmax,
@@ -355,9 +359,14 @@ fbckdengpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NU
 
   if (fixedu) { # fixed threshold (separable) likelihood
     nllh = nlubckdengpd(pvector, u, x, phiu, kernel = kernel,
-      bcmethod = bcmethod, proper = proper, nn = nn, offset = offset, xmax = xmax)
+                        bcmethod = bcmethod, proper = proper, nn = nn, offset = offset, xmax = xmax)
+    if (is.infinite(nllh)) {
+      pvector[3] = 0.1
+      nllh = nlubckdengpd(pvector, u, x, phiu, kernel = kernel,
+                          bcmethod = bcmethod, proper = proper, nn = nn, offset = offset, xmax = xmax)
+    }
     if (is.infinite(nllh)) stop("initial parameter values are invalid")
-  
+
     fit = optim(par = as.vector(pvector), fn = nlubckdengpd, u = u, x = x, phiu = phiu, kernel = kernel,
       bcmethod = bcmethod, proper = proper, nn = nn, offset = offset, xmax = xmax,
       finitelik = finitelik, method = method, control = control, hessian = TRUE, ...)    
@@ -369,7 +378,12 @@ fbckdengpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NU
   } else { # complete (non-separable) likelihood
     
     nllh = nlbckdengpd(pvector, x, phiu, kernel = kernel,
-      bcmethod = bcmethod, proper = proper, nn = nn, offset = offset, xmax = xmax)
+                       bcmethod = bcmethod, proper = proper, nn = nn, offset = offset, xmax = xmax)
+    if (is.infinite(nllh)) {
+      pvector[4] = 0.1
+      nllh = nlbckdengpd(pvector, x, phiu, kernel = kernel,
+                         bcmethod = bcmethod, proper = proper, nn = nn, offset = offset, xmax = xmax)
+    }
     if (is.infinite(nllh)) stop("initial parameter values are invalid")
   
     fit = optim(par = as.vector(pvector), fn = nlbckdengpd, x = x, phiu = phiu, kernel = kernel,
@@ -408,14 +422,14 @@ fbckdengpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NU
   if (std.err) {
     qrhess = qr(fit$hessian)
     if (qrhess$rank != ncol(qrhess$qr)) {
-      warning("observed information matrix is singular; use std.err = FALSE")
+      warning("observed information matrix is singular")
       se = NULL
       invhess = NULL
     } else {
       invhess = solve(qrhess)
       vars = diag(invhess)
       if (any(vars <= 0)) {
-        warning("observed information matrix is singular; use std.err = FALSE")
+        warning("observed information matrix is singular")
         invhess = NULL
         se = NULL
       } else {
@@ -427,8 +441,10 @@ fbckdengpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NU
     se = NULL
   }
   
+  if (!exists("nllhu")) nllhu = NULL
+
   list(call = call, x = as.vector(x),
-    init = as.vector(pvector), fixedu = fixedu, useq = useq,
+    init = as.vector(pvector), fixedu = fixedu, useq = useq, nllhuseq = nllhu,
     optim = fit, conv = conv, cov = invhess, mle = fit$par, se = se, rate = phiu,
     nllh = fit$value, n = n,
     lambda = lambda, u = u, sigmau = sigmau, xi = xi, phiu = phiu, se.phiu = se.phiu, bw = bw, kernel = kernel,
@@ -446,21 +462,20 @@ lbckdengpd <- function(x, lambda = NULL, u = 0, sigmau = 1, xi = 0, phiu = TRUE,
   bcmethod = "simple", proper = TRUE, nn = "jf96", offset = NULL, xmax = NULL, log = TRUE) {
 
   # Check properties of inputs
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
-  check.param(param = lambda, allownull = TRUE) # do not check positivity in likelihood
-  check.param(param = bw, allownull = TRUE)     # do not check positivity in likelihood
-  check.param(param = u)                        # do not check positivity in likelihood
-  check.param(param = sigmau)                   # do not check positivity in likelihood
-  check.param(param = xi)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
+  check.param(lambda, allownull = TRUE)
+  check.param(bw, allownull = TRUE)
+  check.param(u)
+  check.param(sigmau)
+  check.param(xi)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = log)
+  check.logic(log)
 
   check.kernel(kernel)
   check.bcmethod(bcmethod)
-  check.logic(logicarg = proper)
+  check.logic(proper)
   check.nn(nn)
-  check.posparam(offset, allownull = TRUE)  
-  check.offset(offset, bcmethod)
+  check.offset(offset, bcmethod, allowzero = TRUE)
   check.posparam(xmax, allownull = TRUE)  
 
   kernel = ifelse(kernel == "rectangular", "uniform", kernel)
@@ -491,7 +506,7 @@ lbckdengpd <- function(x, lambda = NULL, u = 0, sigmau = 1, xi = 0, phiu = TRUE,
     if (is.null(lambda)) lambda = klambda(bw, kernel)
   }
   
-  check.inputn(c(length(lambda), length(u), length(sigmau), length(xi), length(phiu)))
+  check.inputn(c(length(lambda), length(u), length(sigmau), length(xi), length(phiu)), allowscalar = TRUE)
 
   upboundmethods = c("beta1", "beta2", "copula")
   if (!is.null(xmax) & !(bcmethod %in% upboundmethods))
@@ -515,6 +530,7 @@ lbckdengpd <- function(x, lambda = NULL, u = 0, sigmau = 1, xi = 0, phiu = TRUE,
     }
     # need to recheck there are some valid kernel centres after these exclusions
     check.quant(x)
+    n = length(x)
   }
 
   # assume NA or NaN are irrelevant as entire lower tail is now modelled
@@ -577,16 +593,15 @@ nlbckdengpd <- function(pvector, x, phiu = TRUE, kernel = "gaussian",
 
   # Check properties of inputs
   check.nparam(pvector, nparam = np)
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
 
   check.kernel(kernel)
   check.bcmethod(bcmethod)
-  check.logic(logicarg = proper)
+  check.logic(proper)
   check.nn(nn)
-  check.posparam(offset, allownull = TRUE)  
-  check.offset(offset, bcmethod)
+  check.offset(offset, bcmethod, allowzero = TRUE)
   check.posparam(xmax, allownull = TRUE)  
 
   kernel = ifelse(kernel == "rectangular", "uniform", kernel)
@@ -623,17 +638,18 @@ proflubckdengpd <- function(u, pvector, x, phiu = TRUE, kernel = "gaussian",
   
   # Check properties of inputs
   check.nparam(pvector, nparam = np - 1, allownull = TRUE)
-  check.param(u) # do not check positivity in likelihood
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.posparam(u)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.optim(method)
+  check.control(control)
+  check.logic(finitelik)
 
   check.kernel(kernel)
   check.bcmethod(bcmethod)
-  check.logic(logicarg = proper)
+  check.logic(proper)
   check.nn(nn)
-  check.posparam(offset, allownull = TRUE)  
-  check.offset(offset, bcmethod)
+  check.offset(offset, bcmethod, allowzero = TRUE)
   check.posparam(xmax, allownull = TRUE)  
   
   kernel = ifelse(kernel == "rectangular", "uniform", kernel)
@@ -650,7 +666,6 @@ proflubckdengpd <- function(u, pvector, x, phiu = TRUE, kernel = "gaussian",
   }
 
   check.quant(x)
-  n = length(x)
   
   upboundmethods = c("beta1", "beta2", "copula")
   if (!is.null(xmax) & !(bcmethod %in% upboundmethods))
@@ -682,8 +697,14 @@ proflubckdengpd <- function(u, pvector, x, phiu = TRUE, kernel = "gaussian",
 
   # check initial values for other parameters, try usual alternative
   nllh = nlubckdengpd(pvector, u, x, phiu, kernel = kernel,
-    bcmethod = bcmethod, proper = proper, nn = nn, offset = offset, xmax = xmax)
+                      bcmethod = bcmethod, proper = proper, nn = nn, offset = offset, xmax = xmax)
+  if (is.infinite(nllh)) {
+    pvector[3] = 0.1
+    nllh = nlubckdengpd(pvector, u, x, phiu, kernel = kernel,
+                        bcmethod = bcmethod, proper = proper, nn = nn, offset = offset, xmax = xmax)
+  }
 
+  # if still invalid then output cleanly
   if (is.infinite(nllh)) {
     warning(paste("initial parameter values for threshold u =", u, "are invalid"))
     fit = list(par = rep(NA, np), value = Inf, counts = 0, convergence = NA, 
@@ -717,19 +738,18 @@ nlubckdengpd <- function(pvector, u, x, phiu = TRUE, kernel = "gaussian",
 
   # Check properties of inputs
   check.nparam(pvector, nparam = np - 1)
-  check.param(u) # do not check positivity in likelihood
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.posparam(u)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
 
   check.kernel(kernel)
   check.bcmethod(bcmethod)
-  check.logic(logicarg = proper)
+  check.logic(proper)
   check.nn(nn)
-  check.posparam(offset, allownull = TRUE)  
-  check.offset(offset, bcmethod)
+  check.offset(offset, bcmethod, allowzero = TRUE)
   check.posparam(xmax, allownull = TRUE)  
-
+  
   kernel = ifelse(kernel == "rectangular", "uniform", kernel)
   kernel = ifelse(kernel == "normal", "gaussian", kernel)
     
