@@ -35,25 +35,26 @@
 #'   following elements
 #'
 #' \tabular{ll}{
-#'  \code{call}:    \tab \code{optim} call\cr
-#'  \code{x}:       \tab data vector \code{x}\cr
-#'  \code{init}:    \tab \code{pvector}\cr
-#'  \code{fixedu}:  \tab fixed threshold, logical\cr
-#'  \code{useq}:    \tab threshold vector for profile likelihood or scalar for fixed threshold\cr
-#'  \code{optim}:   \tab complete \code{optim} output\cr
-#'  \code{mle}:     \tab vector of MLE of parameters\cr
-#'  \code{cov}:     \tab variance-covariance matrix of MLE of parameters\cr
-#'  \code{se}:      \tab vector of standard errors of MLE of parameters\cr
-#'  \code{rate}:    \tab \code{phiu} to be consistent with \code{\link[evd:fpot]{evd}}\cr
-#'  \code{nllh}:    \tab minimum negative log-likelihood\cr
-#'  \code{n}:       \tab total sample size\cr
-#'  \code{bshape1}: \tab MLE of beta shape1\cr
-#'  \code{bshape2}: \tab MLE of beta shape2\cr
-#'  \code{u}:       \tab threshold (fixed or MLE)\cr
-#'  \code{sigmau}:  \tab MLE of GPD scale (estimated from other parameters)\cr
-#'  \code{xi}:      \tab MLE of GPD shape\cr
-#'  \code{phiu}:    \tab MLE of tail fraction (bulk model or parameterised approach)\cr
-#'  \code{se.phiu}: \tab standard error of MLE of tail fraction\cr
+#'  \code{call}:      \tab \code{optim} call\cr
+#'  \code{x}:         \tab data vector \code{x}\cr
+#'  \code{init}:      \tab \code{pvector}\cr
+#'  \code{fixedu}:    \tab fixed threshold, logical\cr
+#'  \code{useq}:      \tab threshold vector for profile likelihood or scalar for fixed threshold\cr
+#'  \code{nllhuseq}:  \tab profile negative log-likelihood at each threshold in useq\cr
+#'  \code{optim}:     \tab complete \code{optim} output\cr
+#'  \code{mle}:       \tab vector of MLE of parameters\cr
+#'  \code{cov}:       \tab variance-covariance matrix of MLE of parameters\cr
+#'  \code{se}:        \tab vector of standard errors of MLE of parameters\cr
+#'  \code{rate}:      \tab \code{phiu} to be consistent with \code{\link[evd:fpot]{evd}}\cr
+#'  \code{nllh}:      \tab minimum negative log-likelihood\cr
+#'  \code{n}:         \tab total sample size\cr
+#'  \code{bshape1}:   \tab MLE of beta shape1\cr
+#'  \code{bshape2}:   \tab MLE of beta shape2\cr
+#'  \code{u}:         \tab threshold (fixed or MLE)\cr
+#'  \code{sigmau}:    \tab MLE of GPD scale (estimated from other parameters)\cr
+#'  \code{xi}:        \tab MLE of GPD shape\cr
+#'  \code{phiu}:      \tab MLE of tail fraction (bulk model or parameterised approach)\cr
+#'  \code{se.phiu}:   \tab standard error of MLE of tail fraction\cr
 #' }
 #' 
 #' @note When \code{pvector=NULL} then the initial values are:
@@ -85,8 +86,8 @@
 #' @author Yang Hu and Carl Scarrott \email{carl.scarrott@@canterbury.ac.nz}
 #'
 #' @section Acknowledgments: See Acknowledgments in
-#'   \code{\link[evmix:fnormgpd]{fnormgpd}}, type \code{help fnormgpd}. Based on MATLAB
-#'   code written by Anna MacDonald.
+#'   \code{\link[evmix:fnormgpd]{fnormgpd}}, type \code{help fnormgpd}. Based on code
+#' by Anna MacDonald produced for MATLAB.
 #' 
 #' @seealso \code{\link[stats:Beta]{dbeta}},
 #'  \code{\link[evmix:fgpd]{fgpd}} and \code{\link[evmix:gpd]{gpd}}
@@ -96,7 +97,9 @@
 #' 
 #' @examples
 #' \dontrun{
-#' par(mfrow=c(2,1))
+#' set.seed(1)
+#' par(mfrow = c(2, 1))
+#' 
 #' x = rbeta(1000, shape1 = 2, shape2 = 4)
 #' xx = seq(-0.1, 2, 0.01)
 #' y = dbeta(xx, shape1 = 2, shape2 = 4)
@@ -116,8 +119,8 @@
 #'   col=c("black", "blue", "red"), lty = 1)
 #'   
 #' # Profile likelihood for initial value of threshold and fixed threshold approach
-#' fitu = fbetagpdcon(x, useq = seq(0.5, 0.95, length = 20))
-#' fitfix = fbetagpdcon(x, useq = seq(0.5, 0.95, length = 20), fixedu = TRUE)
+#' fitu = fbetagpdcon(x, useq = seq(0.3, 0.7, length = 20))
+#' fitfix = fbetagpdcon(x, useq = seq(0.3, 0.7, length = 20), fixedu = TRUE)
 #' 
 #' hist(x, breaks = 100, freq = FALSE, xlim = c(-0.1, 2))
 #' lines(xx, y)
@@ -139,16 +142,17 @@ fbetagpdcon <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = N
 
   call <- match.call()
     
+  np = 4 # maximum number of parameters
+
   # Check properties of inputs
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
-  check.logic(logicarg = phiu) # only logical
-  check.posparam(useq, allowvec = TRUE, allownull = TRUE)
-  check.prob(useq, allowvec = TRUE, allownull = TRUE)
-  check.logic(logicarg = fixedu)
-  check.logic(logicarg = std.err)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
+  check.logic(phiu)
+  check.prob(useq, allownull = TRUE)
+  check.logic(fixedu)
+  check.logic(std.err)
   check.optim(method)
   check.control(control)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -162,7 +166,6 @@ fbetagpdcon <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = N
 
   check.quant(x)
   n = length(x)
-  np = 4 # maximum number of parameters
 
   if (any(x > 1)) warning("values greater than one are assumed part of GPD")
 
@@ -190,6 +193,12 @@ fbetagpdcon <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = N
     
   } else { # profile or fixed
     
+    if ((min(useq) <= 0) | (max(useq) >= 1)) {
+      warning("thresholds outside of 0 and 1 (exclusive) are ignored")
+      useq = useq[useq > 0]
+      useq = useq[useq < 1]
+    }
+
     check.nparam(pvector, nparam = np - 1, allownull = TRUE)
 
     # profile likelihood for threshold or scalar given
@@ -197,7 +206,7 @@ fbetagpdcon <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = N
       
       # remove thresholds with less than 5 excesses
       useq = useq[sapply(useq, FUN = function(u, x) sum(x > u) > 5, x = x)]
-      check.param(useq, allowvec = TRUE)
+      check.prob(useq)
       
       nllhu = sapply(useq, proflubetagpdcon, pvector = pvector, x = x, phiu = phiu,
         method = method, control = control, finitelik = finitelik, ...)
@@ -236,6 +245,10 @@ fbetagpdcon <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = N
 
   if (fixedu) { # fixed threshold (separable) likelihood
     nllh = nlubetagpdcon(pvector, u, x, phiu)
+    if (is.infinite(nllh)) {
+      pvector[3] = 0.1
+      nllh = nlubetagpdcon(pvector, u, x, phiu)
+    }
     if (is.infinite(nllh)) stop("initial parameter values are invalid")
   
     fit = optim(par = as.vector(pvector), fn = nlubetagpdcon, u = u, x = x, phiu = phiu,
@@ -248,6 +261,10 @@ fbetagpdcon <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = N
   } else { # complete (non-separable) likelihood
     
     nllh = nlbetagpdcon(pvector, x, phiu)
+    if (is.infinite(nllh)) {
+      pvector[4] = 0.1
+      nllh = nlbetagpdcon(pvector, x, phiu)
+    }
     if (is.infinite(nllh)) stop("initial parameter values are invalid")
   
     fit = optim(par = as.vector(pvector), fn = nlbetagpdcon, x = x, phiu = phiu,
@@ -281,14 +298,14 @@ fbetagpdcon <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = N
   if (std.err) {
     qrhess = qr(fit$hessian)
     if (qrhess$rank != ncol(qrhess$qr)) {
-      warning("observed information matrix is singular; use std.err = FALSE")
+      warning("observed information matrix is singular")
       se = NULL
       invhess = NULL
     } else {
       invhess = solve(qrhess)
       vars = diag(invhess)
       if (any(vars <= 0)) {
-        warning("observed information matrix is singular; use std.err = FALSE")
+        warning("observed information matrix is singular")
         invhess = NULL
         se = NULL
       } else {
@@ -300,8 +317,10 @@ fbetagpdcon <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = N
     se = NULL
   }
   
+  if (!exists("nllhu")) nllhu = NULL
+
   list(call = call, x = as.vector(x), 
-    init = as.vector(pvector), fixedu = fixedu, useq = useq,
+    init = as.vector(pvector), fixedu = fixedu, useq = useq, nllhuseq = nllhu,
     optim = fit, conv = conv, cov = invhess, mle = fit$par, se = se, rate = phiu,
     nllh = fit$value, n = n,
     bshape1 = bshape1, bshape2 = bshape2, u = u, sigmau = sigmau, xi = xi, phiu = phiu, se.phiu = se.phiu)
@@ -316,13 +335,13 @@ lbetagpdcon <- function(x, bshape1 = 1, bshape2 = 1, u = qbeta(0.9, bshape1, bsh
   xi = 0, phiu = TRUE, log = TRUE) {
 
   # Check properties of inputs
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
-  check.param(param = bshape1) # do not check positivity in likelihood
-  check.param(param = bshape2) # do not check positivity in likelihood
-  check.param(param = u)       # do not check probability in likelihood
-  check.param(param = xi)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
+  check.param(bshape1)
+  check.param(bshape2)
+  check.param(u)
+  check.param(xi)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = log)
+  check.logic(log)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -337,14 +356,14 @@ lbetagpdcon <- function(x, bshape1 = 1, bshape2 = 1, u = qbeta(0.9, bshape1, bsh
   if (any(x > 1)) warning("values greater than one are assumed part of GPD")
 
   check.quant(x)
+  n = length(x)
 
-  check.inputn(c(length(bshape1), length(bshape2), length(u), length(xi), length(phiu)))
+  check.inputn(c(length(bshape1), length(bshape2), length(u), length(xi), length(phiu)), allowscalar = TRUE)
 
   # assume NA or NaN are irrelevant as entire lower tail is now modelled
   # inconsistent with evd library definition
   # hence use which() to ignore these
 
-  n = length(x)
   xu = x[which(x > u)]
   nu = length(xu)
   xb = x[which(x <= u)]
@@ -397,9 +416,9 @@ nlbetagpdcon <- function(pvector, x, phiu = TRUE, finitelik = FALSE) {
 
   # Check properties of inputs
   check.nparam(pvector, nparam = np)
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
 
   bshape1 = pvector[1]
   bshape2 = pvector[2]
@@ -429,10 +448,13 @@ proflubetagpdcon <- function(u, pvector, x, phiu = TRUE, method = "BFGS",
   
   # Check properties of inputs
   check.nparam(pvector, nparam = np - 1, allownull = TRUE)
-  check.param(u) # do not check probability in likelihood
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.posparam(u)
+  check.prob(u)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.optim(method)
+  check.control(control)
+  check.logic(finitelik)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -448,6 +470,9 @@ proflubetagpdcon <- function(u, pvector, x, phiu = TRUE, method = "BFGS",
 
   check.quant(x)
 
+  if ((u <= 0) | (u >= 1))
+    stop("threshold must be between 0 and 1 (exclusive)")
+
   # check initial values for other parameters, try usual alternative
   if (!is.null(pvector)) {
     nllh = nlubetagpdcon(pvector, u, x, phiu)
@@ -463,7 +488,12 @@ proflubetagpdcon <- function(u, pvector, x, phiu = TRUE, method = "BFGS",
     initfgpd = fgpd(x, u, std.err = FALSE)
     pvector[3] = initfgpd$xi
     nllh = nlubetagpdcon(pvector, u, x, phiu)
-  }  
+  }
+
+  if (is.infinite(nllh)) {
+    pvector[3] = 0.1
+    nllh = nlubetagpdcon(pvector, u, x, phiu)
+  }
 
   # if still invalid then output cleanly
   if (is.infinite(nllh)) {
@@ -495,11 +525,15 @@ nlubetagpdcon <- function(pvector, u, x, phiu = TRUE, finitelik = FALSE) {
 
   # Check properties of inputs
   check.nparam(pvector, nparam = np - 1)
-  check.param(u) # do not check probability in likelihood
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.posparam(u)
+  check.prob(u)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
     
+  if ((u <= 0) | (u >= 1))
+    stop("threshold must be between 0 and 1 (exclusive)")
+
   bshape1 = pvector[1]
   bshape2 = pvector[2]
   xi = pvector[3]
