@@ -34,25 +34,26 @@
 #'   following elements
 #'
 #' \tabular{ll}{
-#'  \code{call}:    \tab \code{optim} call\cr
-#'  \code{x}:       \tab data vector \code{x}\cr
-#'  \code{init}:    \tab \code{pvector}\cr
-#'  \code{fixedu}:  \tab fixed threshold, logical\cr
-#'  \code{useq}:    \tab threshold vector for profile likelihood or scalar for fixed threshold\cr
-#'  \code{optim}:   \tab complete \code{optim} output\cr
-#'  \code{mle}:     \tab vector of MLE of parameters\cr
-#'  \code{cov}:     \tab variance-covariance matrix of MLE of parameters\cr
-#'  \code{se}:      \tab vector of standard errors of MLE of parameters\cr
-#'  \code{rate}:    \tab \code{phiu} to be consistent with \code{\link[evd:fpot]{evd}}\cr
-#'  \code{nllh}:    \tab minimum negative log-likelihood\cr
-#'  \code{n}:       \tab total sample size\cr
-#'  \code{gshape}:  \tab MLE of gamma shape\cr
-#'  \code{gscale}:  \tab MLE of gamma scale\cr
-#'  \code{u}:       \tab threshold (fixed or MLE)\cr
-#'  \code{sigmau}:  \tab MLE of GPD scale\cr
-#'  \code{xi}:      \tab MLE of GPD shape\cr
-#'  \code{phiu}:    \tab MLE of tail fraction (bulk model or parameterised approach)\cr
-#'  \code{se.phiu}: \tab standard error of MLE of tail fraction\cr
+#'  \code{call}:      \tab \code{optim} call\cr
+#'  \code{x}:         \tab data vector \code{x}\cr
+#'  \code{init}:      \tab \code{pvector}\cr
+#'  \code{fixedu}:    \tab fixed threshold, logical\cr
+#'  \code{useq}:      \tab threshold vector for profile likelihood or scalar for fixed threshold\cr
+#'  \code{nllhuseq}:  \tab profile negative log-likelihood at each threshold in useq\cr
+#'  \code{optim}:     \tab complete \code{optim} output\cr
+#'  \code{mle}:       \tab vector of MLE of parameters\cr
+#'  \code{cov}:       \tab variance-covariance matrix of MLE of parameters\cr
+#'  \code{se}:        \tab vector of standard errors of MLE of parameters\cr
+#'  \code{rate}:      \tab \code{phiu} to be consistent with \code{\link[evd:fpot]{evd}}\cr
+#'  \code{nllh}:      \tab minimum negative log-likelihood\cr
+#'  \code{n}:         \tab total sample size\cr
+#'  \code{gshape}:    \tab MLE of gamma shape\cr
+#'  \code{gscale}:    \tab MLE of gamma scale\cr
+#'  \code{u}:         \tab threshold (fixed or MLE)\cr
+#'  \code{sigmau}:    \tab MLE of GPD scale\cr
+#'  \code{xi}:        \tab MLE of GPD shape\cr
+#'  \code{phiu}:      \tab MLE of tail fraction (bulk model or parameterised approach)\cr
+#'  \code{se.phiu}:   \tab standard error of MLE of tail fraction\cr
 #' }
 #' 
 #' @note When \code{pvector=NULL} then the initial values are:
@@ -95,7 +96,9 @@
 #' 
 #' @examples
 #' \dontrun{
-#' par(mfrow=c(2,1))
+#' set.seed(1)
+#' par(mfrow = c(2, 1))
+#' 
 #' x = rgamma(1000, shape = 2)
 #' xx = seq(-0.1, 8, 0.01)
 #' y = dgamma(xx, shape = 2)
@@ -138,15 +141,17 @@ fgammagpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NUL
 
   call <- match.call()
     
+  np = 5 # maximum number of parameters
+
   # Check properties of inputs
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
-  check.logic(logicarg = phiu) # only logical
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
+  check.logic(phiu)
   check.posparam(useq, allowvec = TRUE, allownull = TRUE)
-  check.logic(logicarg = fixedu)
-  check.logic(logicarg = std.err)
+  check.logic(fixedu)
+  check.logic(std.err)
   check.optim(method)
   check.control(control)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -160,7 +165,6 @@ fgammagpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NUL
 
   check.quant(x)
   n = length(x)
-  np = 5 # maximum number of parameters
 
   if ((method == "L-BFGS-B") | (method == "BFGS")) finitelik = TRUE
   
@@ -194,7 +198,7 @@ fgammagpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NUL
       
       # remove thresholds with less than 5 excesses
       useq = useq[sapply(useq, FUN = function(u, x) sum(x > u) > 5, x = x)]
-      check.param(useq, allowvec = TRUE)
+      check.posparam(useq, allowvec = TRUE)
       
       nllhu = sapply(useq, proflugammagpd, pvector = pvector, x = x, phiu = phiu,
         method = method, control = control, finitelik = finitelik, ...)
@@ -236,6 +240,10 @@ fgammagpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NUL
 
   if (fixedu) { # fixed threshold (separable) likelihood
     nllh = nlugammagpd(pvector, u, x, phiu)
+    if (is.infinite(nllh)) {
+      pvector[4] = 0.1
+      nllh = nlugammagpd(pvector, u, x, phiu)
+    }    
     if (is.infinite(nllh)) stop("initial parameter values are invalid")
   
     fit = optim(par = as.vector(pvector), fn = nlugammagpd, u = u, x = x, phiu = phiu,
@@ -249,6 +257,10 @@ fgammagpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NUL
   } else { # complete (non-separable) likelihood
     
     nllh = nlgammagpd(pvector, x, phiu)
+    if (is.infinite(nllh)) {
+      pvector[5] = 0.1
+      nllh = nlgammagpd(pvector, x, phiu)
+    }    
     if (is.infinite(nllh)) stop("initial parameter values are invalid")
   
     fit = optim(par = as.vector(pvector), fn = nlgammagpd, x = x, phiu = phiu,
@@ -279,14 +291,14 @@ fgammagpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NUL
   if (std.err) {
     qrhess = qr(fit$hessian)
     if (qrhess$rank != ncol(qrhess$qr)) {
-      warning("observed information matrix is singular; use std.err = FALSE")
+      warning("observed information matrix is singular")
       se = NULL
       invhess = NULL
     } else {
       invhess = solve(qrhess)
       vars = diag(invhess)
       if (any(vars <= 0)) {
-        warning("observed information matrix is singular; use std.err = FALSE")
+        warning("observed information matrix is singular")
         invhess = NULL
         se = NULL
       } else {
@@ -298,8 +310,10 @@ fgammagpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NUL
     se = NULL
   }
   
+  if (!exists("nllhu")) nllhu = NULL
+
   list(call = call, x = as.vector(x), 
-    init = as.vector(pvector), fixedu = fixedu, useq = useq,
+    init = as.vector(pvector), fixedu = fixedu, useq = useq, nllhuseq = nllhu,
     optim = fit, conv = conv, cov = invhess, mle = fit$par, se = se, rate = phiu,
     nllh = fit$value, n = n,
     gshape = gshape, gscale = gscale, u = u, sigmau = sigmau, xi = xi, phiu = phiu, se.phiu = se.phiu)
@@ -314,14 +328,14 @@ lgammagpd <- function(x, gshape = 1, gscale = 1, u = qgamma(0.9, gshape, 1/gscal
   sigmau = sqrt(gshape) * gscale, xi = 0, phiu = TRUE, log = TRUE) {
 
   # Check properties of inputs
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
-  check.param(param = gshape) # do not check positivity in likelihood
-  check.param(param = gscale) # do not check positivity in likelihood
-  check.param(param = u)      # do not check positivity in likelihood
-  check.param(param = sigmau) # do not check positivity in likelihood
-  check.param(param = xi)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
+  check.param(gshape)
+  check.param(gscale)
+  check.param(u)
+  check.param(sigmau)
+  check.param(xi)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = log)
+  check.logic(log)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -334,14 +348,15 @@ lgammagpd <- function(x, gshape = 1, gscale = 1, u = qgamma(0.9, gshape, 1/gscal
   }
 
   check.quant(x)
+  n = length(x)
 
-  check.inputn(c(length(gshape), length(gscale), length(u), length(sigmau), length(xi), length(phiu)))
+  check.inputn(c(length(gshape), length(gscale), length(u), length(sigmau), length(xi), length(phiu)),
+               allowscalar = TRUE)
 
   # assume NA or NaN are irrelevant as entire lower tail is now modelled
   # inconsistent with evd library definition
   # hence use which() to ignore these
 
-  n = length(x)
   xu = x[which(x > u)]
   nu = length(xu)
   xb = x[which(x <= u)]
@@ -391,9 +406,9 @@ nlgammagpd <- function(pvector, x, phiu = TRUE, finitelik = FALSE) {
 
   # Check properties of inputs
   check.nparam(pvector, nparam = np)
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
 
   gshape = pvector[1]
   gscale = pvector[2]
@@ -424,10 +439,12 @@ proflugammagpd <- function(u, pvector, x, phiu = TRUE, method = "BFGS",
 
   # Check properties of inputs
   check.nparam(pvector, nparam = np - 1, allownull = TRUE)
-  check.param(u) # do not check positivity in likelihood
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.posparam(u)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.optim(method)
+  check.control(control)
+  check.logic(finitelik)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -459,6 +476,11 @@ proflugammagpd <- function(u, pvector, x, phiu = TRUE, method = "BFGS",
     nllh = nlugammagpd(pvector, u, x, phiu)
   }  
 
+  if (is.infinite(nllh)) {
+    pvector[4] = 0.1
+    nllh = nlugammagpd(pvector, u, x, phiu)
+  }    
+
   # if still invalid then output cleanly
   if (is.infinite(nllh)) {
     warning(paste("initial parameter values for threshold u =", u, "are invalid"))
@@ -489,10 +511,10 @@ nlugammagpd <- function(pvector, u, x, phiu = TRUE, finitelik = FALSE) {
 
   # Check properties of inputs
   check.nparam(pvector, nparam = np - 1)
-  check.param(u) # do not check positivity in likelihood
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.posparam(u)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
     
   gshape = pvector[1]
   gscale = pvector[2]
