@@ -31,6 +31,9 @@
 #' (\code{nmean}, \code{nsd}, \code{xil}, \code{xir})
 #' for profile likelihood or fixed threshold approach.
 #' 
+#' If the profile likelihood approach is used, then a grid search over all combinations of both thresholds
+#' is carried out. The combinations which lead to less than 5 in any datapoints beyond the thresholds are not considered.
+#' 
 #' @return Log-likelihood is given by \code{\link[evmix:fgngcon]{lgngcon}} and it's
 #'   wrappers for negative log-likelihood from \code{\link[evmix:fgngcon]{nlgngcon}}
 #'   and \code{\link[evmix:fgngcon]{nlugngcon}}. Profile likelihood for both
@@ -45,6 +48,7 @@
 #'  \code{fixedu}:    \tab fixed thresholds, logical\cr
 #'  \code{ulseq}:     \tab lower threshold vector for profile likelihood or scalar for fixed threshold\cr
 #'  \code{urseq}:     \tab upper threshold vector for profile likelihood or scalar for fixed threshold\cr
+#'  \code{nllhuseq}:  \tab profile negative log-likelihood at each threshold pair in (ulseq, urseq)\cr
 #'  \code{optim}:     \tab complete \code{optim} output\cr
 #'  \code{mle}:       \tab vector of MLE of parameters\cr
 #'  \code{cov}:       \tab variance-covariance matrix of MLE of parameters\cr
@@ -98,8 +102,8 @@
 #' @author Yang Hu and Carl Scarrott \email{carl.scarrott@@canterbury.ac.nz}
 #'
 #' @section Acknowledgments: See Acknowledgments in
-#'   \code{\link[evmix:fnormgpd]{fnormgpd}}, type \code{help fnormgpd}. Based on MATLAB
-#'   code written by Xin Zhao.
+#'   \code{\link[evmix:fnormgpd]{fnormgpd}}, type \code{help fnormgpd}. Based on code
+#' by Xin Zhao produced for MATLAB.
 #' 
 #' @seealso \code{\link[stats:Normal]{dnorm}},
 #'  \code{\link[evmix:fgpd]{fgpd}} and \code{\link[evmix:gpd]{gpd}}
@@ -109,7 +113,9 @@
 #' 
 #' @examples
 #' \dontrun{
-#' par(mfrow=c(2,1))
+#' set.seed(1)
+#' par(mfrow = c(2, 1))
+#' 
 #' x = rnorm(1000)
 #' xx = seq(-4, 4, 0.01)
 #' y = dnorm(xx)
@@ -131,10 +137,10 @@
 #'   col=c("black", "blue", "red"), lty = 1)
 #'   
 #' # Profile likelihood for initial value of threshold and fixed threshold approach
-#' fitu = fgngcon(x, ulseq = rep(seq(-2, -0.2, length = 10), times = 10), 
-#'  urseq = rep(seq(0.2, 2, length = 10), each = 10))
-#' fitfix = fgngcon(x, ulseq = rep(seq(-2, -0.2, length = 10), times = 10), 
-#'  urseq = rep(seq(0.2, 2, length = 10), each = 10), fixedu = TRUE)
+#' fitu = fgngcon(x, ulseq = seq(-2, -0.2, length = 10), 
+#'  urseq = seq(0.2, 2, length = 10))
+#' fitfix = fgngcon(x, ulseq = seq(-2, -0.2, length = 10), 
+#'  urseq = seq(0.2, 2, length = 10), fixedu = TRUE)
 #' 
 #' hist(x, breaks = 100, freq = FALSE, xlim = c(-4, 4))
 #' lines(xx, y)
@@ -160,17 +166,19 @@ fgngcon <- function(x, phiul = TRUE, phiur = TRUE, ulseq = NULL, urseq = NULL, f
 
   call <- match.call()
     
+  np = 6 # maximum number of parameters
+
   # Check properties of inputs
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
-  check.logic(logicarg = phiul) # only logical
-  check.logic(logicarg = phiur) # only logical
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
+  check.logic(phiul)
+  check.logic(phiur)
   check.param(ulseq, allowvec = TRUE, allownull = TRUE)
   check.param(urseq, allowvec = TRUE, allownull = TRUE)
-  check.logic(logicarg = fixedu)
-  check.logic(logicarg = std.err)
+  check.logic(fixedu)
+  check.logic(std.err)
   check.optim(method)
   check.control(control)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -179,7 +187,6 @@ fgngcon <- function(x, phiul = TRUE, phiur = TRUE, ulseq = NULL, urseq = NULL, f
 
   check.quant(x)
   n = length(x)
-  np = 6 # maximum number of parameters
 
   if ((method == "L-BFGS-B") | (method == "BFGS")) finitelik = TRUE
   
@@ -215,20 +222,25 @@ fgngcon <- function(x, phiul = TRUE, phiur = TRUE, ulseq = NULL, urseq = NULL, f
       ulseq = ulseq[sapply(ulseq, FUN = function(u, x) sum(x < u) > 5, x = x)]
       check.param(ulseq, allowvec = TRUE)
       urseq = urseq[sapply(urseq, FUN = function(u, x) sum(x > u) > 5, x = x)]
-      check.param(ulseq, allowvec = TRUE)
+      check.param(urseq, allowvec = TRUE)
 
-      nuseq = max(length(ulseq), length(urseq))
-      ulseq = rep(ulseq, length.out = nuseq)
-      urseq = rep(urseq, length.out = nuseq)
+      ulrseq = expand.grid(ulseq, urseq)
       
-      nllhu = apply(cbind(ulseq, urseq), 1, proflugngcon, pvector = pvector, x = x,
+      # remove those where ulseq >= urseq
+      if (any(ulrseq[1] >= ulrseq[2])) {
+        warning("lower thresholds above or equal to upper threshold are ignored")
+        ulrseq = ulrseq[ulrseq[1] < ulrseq[2],]
+      }
+
+      nllhu = apply(ulrseq, 1, proflugngcon, pvector = pvector, x = x,
         phiul = phiul, phiur = phiur, method = method, control = control, finitelik = finitelik, ...)
       
       if (all(!is.finite(nllhu))) stop("thresholds are all invalid")
-      ul = ulseq[which.min(nllhu)]
-      ur = urseq[which.min(nllhu)]
+      ul = ulrseq[which.min(nllhu), 1]
+      ur = ulrseq[which.min(nllhu), 2]
 
     } else {
+      if (ulseq >= urseq) stop("lower threshold cannot be above or equal to upper threshold")
       ul = ulseq
       ur = urseq
     }
@@ -263,6 +275,14 @@ fgngcon <- function(x, phiul = TRUE, phiur = TRUE, ulseq = NULL, urseq = NULL, f
 
   if (fixedu) { # fixed threshold (separable) likelihood
     nllh = nlugngcon(pvector, ul, ur, x, phiul, phiur)
+    
+    # if initial parameter vector is invalid then try xi=0.1 (shape parameter is common problem)
+    if (is.infinite(nllh)) {
+      pvector[3] = 0.1
+      pvector[4] = 0.1
+      nllh = nlugngcon(pvector, ul, ur, x, phiul, phiur)
+    }
+    
     if (is.infinite(nllh)) stop("initial parameter values are invalid")
   
     fit = optim(par = as.vector(pvector), fn = nlugngcon, ul = ul, ur = ur, x = x,
@@ -276,6 +296,14 @@ fgngcon <- function(x, phiul = TRUE, phiur = TRUE, ulseq = NULL, urseq = NULL, f
   } else { # complete (non-separable) likelihood
     
     nllh = nlgngcon(pvector, x, phiul, phiur)
+    
+    # if initial parameter vector is invalid then try xi=0.1 (shape parameter is common problem)
+    if (is.infinite(nllh)) {
+      pvector[4] = 0.1
+      pvector[6] = 0.1
+      nllh = nlgngcon(pvector, x, phiul, phiur)
+    }
+
     if (is.infinite(nllh)) stop("initial parameter values are invalid")
   
     fit = optim(par = as.vector(pvector), fn = nlgngcon, x = x, phiul = phiul, phiur = phiur,
@@ -322,14 +350,14 @@ fgngcon <- function(x, phiul = TRUE, phiur = TRUE, ulseq = NULL, urseq = NULL, f
   if (std.err) {
     qrhess = qr(fit$hessian)
     if (qrhess$rank != ncol(qrhess$qr)) {
-      warning("observed information matrix is singular; use std.err = FALSE")
+      warning("observed information matrix is singular")
       se = NULL
       invhess = NULL
     } else {
       invhess = solve(qrhess)
       vars = diag(invhess)
       if (any(vars <= 0)) {
-        warning("observed information matrix is singular; use std.err = FALSE")
+        warning("observed information matrix is singular")
         invhess = NULL
         se = NULL
       } else {
@@ -341,8 +369,10 @@ fgngcon <- function(x, phiul = TRUE, phiur = TRUE, ulseq = NULL, urseq = NULL, f
     se = NULL
   }
   
+  if (!exists("nllhu")) nllhu = NULL
+
   list(call = call, x = as.vector(x),
-    init = as.vector(pvector), fixedu = fixedu, ulseq = ulseq, urseq = urseq,
+    init = as.vector(pvector), fixedu = fixedu, ulseq = ulseq, urseq = urseq, nllhuseq = nllhu,
     optim = fit, conv = conv, cov = invhess, mle = fit$par, se = se, ratel = phiul, rater = phiur,
     nllh = fit$value, n = n, nmean = nmean, nsd = nsd,
     ul = ul, sigmaul = sigmaul, xil = xil, phiul = phiul, se.phiul = se.phiul, 
@@ -359,16 +389,16 @@ lgngcon <- function(x, nmean = 0, nsd = 1,
   ul = 0, xil = 0, phiul = TRUE, ur = 0, xir = 0, phiur = TRUE, log = TRUE) {
 
   # Check properties of inputs
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
-  check.param(param = nmean)
-  check.param(param = nsd)     # do not check positivity in likelihood
-  check.param(param = ul)                       
-  check.param(param = xil)
-  check.param(param = ur)                       
-  check.param(param = xir)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
+  check.param(nmean)
+  check.param(nsd)
+  check.param(ul)                       
+  check.param(xil)
+  check.param(ur)                       
+  check.param(xir)
   check.phiu(phiul, allowfalse = TRUE)
   check.phiu(phiur, allowfalse = TRUE)
-  check.logic(logicarg = log)
+  check.logic(log)
   
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -379,7 +409,7 @@ lgngcon <- function(x, nmean = 0, nsd = 1,
   n = length(x)
   
   check.inputn(c(length(nmean), length(nsd), 
-    length(ul), length(xil), length(phiul), length(ur), length(xir), length(phiur)))
+    length(ul), length(xil), length(phiul), length(ur), length(xir), length(phiur)), allowscalar = TRUE)
 
   # assume NA or NaN are irrelevant as entire lower tail is now modelled
   # inconsistent with evd library definition
@@ -453,10 +483,10 @@ nlgngcon <- function(pvector, x, phiul = TRUE, phiur = TRUE, finitelik = FALSE)
 
   # Check properties of inputs
   check.nparam(pvector, nparam = np)
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiul, allowfalse = TRUE)
   check.phiu(phiur, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
 
   nmean = pvector[1]
   nsd = pvector[2]
@@ -490,10 +520,12 @@ proflugngcon <- function(ulr, pvector, x, phiul = TRUE, phiur = TRUE,
   check.nparam(pvector, nparam = np - 2, allownull = TRUE)
   check.param(ulr, allowvec = TRUE)
   check.nparam(ulr, nparam = 2)
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiul, allowfalse = TRUE)
   check.phiu(phiur, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.optim(method)
+  check.control(control)
+  check.logic(finitelik)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -501,11 +533,12 @@ proflugngcon <- function(ulr, pvector, x, phiul = TRUE, phiur = TRUE,
   }
 
   check.quant(x)
-  n = length(x)
   
   ul = ulr[1]
   ur = ulr[2]
 
+  if (ul >= ur) stop("lower threshold cannot be above or equal to upper threshold")
+
   # check initial values for other parameters, try usual alternative
   if (!is.null(pvector)) {
     nllh = nlugngcon(pvector, ul, ur, x, phiul, phiur)
@@ -523,6 +556,13 @@ proflugngcon <- function(ulr, pvector, x, phiul = TRUE, phiur = TRUE,
     nllh = nlugngcon(pvector, ul, ur, x, phiul, phiur)
   }
 
+  # if initial parameter vector is invalid then try xi=0.1 (shape parameter is common problem)
+  if (is.infinite(nllh)) {
+    pvector[3] = 0.1
+    pvector[4] = 0.1
+    nllh = nlugngcon(pvector, ul, ur, x, phiul, phiur)
+  }
+
   # if still invalid then output cleanly
   if (is.infinite(nllh)) {
     warning(paste("initial parameter values for thresholds ul =", ul, "and ur =", ur,"are invalid"))
@@ -556,11 +596,13 @@ nlugngcon <- function(pvector, ul, ur, x, phiul = TRUE, phiur = TRUE, finitelik
   check.nparam(pvector, nparam = np - 2)
   check.param(ul)
   check.param(ur)
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiul, allowfalse = TRUE)
   check.phiu(phiur, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
     
+  if (ul >= ur) stop("lower threshold cannot be above or equal to upper threshold")
+
   nmean = pvector[1]
   nsd = pvector[2]
   xil = pvector[3]
