@@ -104,6 +104,7 @@
 #'  \code{init}:      \tab \code{pvector}\cr
 #'  \code{fixedu}:    \tab fixed threshold, logical\cr
 #'  \code{useq}:      \tab threshold vector for profile likelihood or scalar for fixed threshold\cr
+#'  \code{nllhuseq}:  \tab profile negative log-likelihood at each threshold in useq\cr
 #'  \code{optim}:     \tab complete \code{optim} output\cr
 #'  \code{mle}:       \tab vector of MLE of parameters\cr
 #'  \code{cov}:       \tab variance-covariance matrix of MLE of parameters\cr
@@ -165,7 +166,9 @@
 #' 
 #' @author Carl Scarrott \email{carl.scarrott@@canterbury.ac.nz}
 #'
-#' @section Acknowledgments: See Acknowledgments in
+#' @section Acknowledgments: Thanks to Daniela Laas, University of St Gallen, Switzerland for reporting various bugs in these functions.
+#' 
+#' See Acknowledgments in
 #'   \code{\link[evmix:fnormgpd]{fnormgpd}}, type \code{help fnormgpd}.
 #' 
 #' @seealso \code{\link[stats:GammaDist]{dgamma}},
@@ -178,7 +181,9 @@
 #' 
 #' @examples
 #' \dontrun{
-#' par(mfrow=c(2,1))
+#' set.seed(1)
+#' par(mfrow = c(2, 1))
+#' 
 #' n=1000
 #' x = c(rgamma(n*0.25, shape = 1, scale = 1), rgamma(n*0.75, shape = 6, scale = 2))
 #' xx = seq(-1, 40, 0.01)
@@ -223,18 +228,19 @@ fmgammagpdcon <- function(x, M, phiu = TRUE, useq = NULL, fixedu = FALSE, pvecto
 
   call <- match.call()
     
+  check.n(M)
+  if (M == 1) stop("use fgammagpdcon instead")
+  np = 3*M + 2 # maximum number of parameters
+
   # Check properties of inputs
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
-  check.logic(logicarg = phiu)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
+  check.logic(phiu)
   check.posparam(useq, allowvec = TRUE, allownull = TRUE)
-  check.logic(logicarg = fixedu)
-  check.logic(logicarg = std.err)
+  check.logic(fixedu)
+  check.logic(std.err)
   check.optim(method)
   check.control(control)
-  check.logic(logicarg = finitelik)
-
-  check.n(M)
-  if (M == 1) stop("use fgammagpd instead")
+  check.logic(finitelik)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -248,7 +254,6 @@ fmgammagpdcon <- function(x, M, phiu = TRUE, useq = NULL, fixedu = FALSE, pvecto
 
   check.quant(x)
   n = length(x)
-  np = 3*M + 2 # maximum number of parameters
 
   if (is.unsorted(x)) x = sort(x)
 
@@ -304,7 +309,7 @@ fmgammagpdcon <- function(x, M, phiu = TRUE, useq = NULL, fixedu = FALSE, pvecto
       
       # remove thresholds with less than 5 excesses
       useq = useq[sapply(useq, FUN = function(u, x) sum(x > u) > 5, x = x)]
-      check.param(useq, allowvec = TRUE)
+      check.posparam(useq, allowvec = TRUE)
       
       nllhu = sapply(useq, proflumgammagpdcon, pvector = pvector, x = x, M = M, phiu = phiu,
         method = method, control = control, finitelik = finitelik, ...)
@@ -414,10 +419,9 @@ fmgammagpdcon <- function(x, M, phiu = TRUE, useq = NULL, fixedu = FALSE, pvecto
       i = i + 1
       
       i.pu = pmgamma(u, mgshape, mgscale, mgweight)
-      if (is.logical(phiu)) {
+      if (phiu) {
         i.phiu = 1 - i.pu
-      }
-      else {
+      } else {
         i.phiu = mean(x > u)
       }
       i.phib = (1 - i.phiu) / i.pu
@@ -498,21 +502,19 @@ fmgammagpdcon <- function(x, M, phiu = TRUE, useq = NULL, fixedu = FALSE, pvecto
     # approximate negative log-likelihood at initial step
     i = 1
     nllh[i + 1] = nlmgammagpdcon(pvector, x, M, phiu)  
-    
+
     if (is.infinite(nllh[i + 1])) stop("initial parameter values are invalid")
     
     EMresults[i,] = c(nllh[i + 1], mgshape, mgscale, mgweight, u, xi, NA, NA)
 
-
     while ((abs(nllh[i + 1] - nllh[i]) > abstol) & (i < maxit)) {
-      
+
       i = i + 1
       
       i.pu = pmgamma(u, mgshape, mgscale, mgweight)
-      if (is.logical(phiu)) {
+      if (phiu) {
         i.phiu = 1 - i.pu
-      }
-      else {
+      } else {
         i.phiu = mean(x > u)
       }
       i.phib = (1 - i.phiu) / i.pu
@@ -526,13 +528,13 @@ fmgammagpdcon <- function(x, M, phiu = TRUE, useq = NULL, fixedu = FALSE, pvecto
       
       # unweighted density from each component
       Mdens = sapply(1:M, FUN = function(i) dgammagpd(x, mgshape[[i]], mgscale[[i]], u, sigmau, xi))
-      
+
       # Reapply relative scaling phiu and phib
       Mdens[x > u,] = Mdens[x > u,] * i.phiu / (1 - i.pu)
       Mdens[x <= u,] = Mdens[x <= u,] * i.phib
       
       Mprob = t(mgweight * t(Mdens))
-      
+
       # Estimated (posterior) probability of individual data being from each component,
       # conditional on each components' parameters
       tau = Mprob / rowSums(Mprob)
@@ -547,7 +549,7 @@ fmgammagpdcon <- function(x, M, phiu = TRUE, useq = NULL, fixedu = FALSE, pvecto
       fit = try(optim(par = c(mgshape, mgscale, u, xi), fn = nlEMmgammagpdcon,
         tau = tau, mgweight = mgweight, x = x, M = M, phiu = phiu,
         finitelik = finitelik, method = method, control = control, hessian = TRUE, ...))
-      
+            
       if (inherits(fit, "try-error")) stop("Maximisation step failed, try new initial values")
       
       conv = TRUE
@@ -604,14 +606,14 @@ fmgammagpdcon <- function(x, M, phiu = TRUE, useq = NULL, fixedu = FALSE, pvecto
   if (std.err) {
     qrhess = qr(fit$hessian)
     if (qrhess$rank != ncol(qrhess$qr)) {
-      warning("observed information matrix is singular; use std.err = FALSE")
+      warning("observed information matrix is singular")
       se = NULL
       invhess = NULL
     } else {
       invhess = solve(qrhess)
       vars = diag(invhess)
       if (any(vars <= 0)) {
-        warning("observed information matrix is singular; use std.err = FALSE")
+        warning("observed information matrix is singular")
         invhess = NULL
         se = NULL
       } else {
@@ -623,7 +625,9 @@ fmgammagpdcon <- function(x, M, phiu = TRUE, useq = NULL, fixedu = FALSE, pvecto
     se = NULL
   }
   
-  list(call = call, x = as.vector(x), init = as.vector(pvector), fixedu = fixedu, useq = useq,
+  if (!exists("nllhu")) nllhu = NULL
+  
+  list(call = call, x = as.vector(x), init = as.vector(pvector), fixedu = fixedu, useq = useq, nllhuseq = nllhu,
     optim = fit, conv = conv, cov = invhess, mle = fit$par, se = se, rate = phiu,
     nllh = nllh[i + 1], n = n, M = M, mgshape = mgshape, mgscale = mgscale, mgweight = mgweight,
     u = u, sigmau = sigmau, xi = xi, phiu = phiu, se.phiu = se.phiu, EMresults = EMresults, posterior = tau)
@@ -637,23 +641,23 @@ fmgammagpdcon <- function(x, M, phiu = TRUE, useq = NULL, fixedu = FALSE, pvecto
 lmgammagpdcon <- function(x, mgshape, mgscale, mgweight, u, xi, phiu = TRUE, log = TRUE) {
   
   # Check properties of inputs
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
-  check.param(param = u)       # do not check positivity in likelihood
-  check.param(param = xi)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
+  check.param(u)
+  check.param(xi)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = log)
+  check.logic(log)
 
   # user may try to input lists for mixture of gammas parameter vectors
   if (is.list(mgshape)) mgshape = unlist(mgshape)
   if (is.list(mgscale)) mgscale = unlist(mgscale)
   if (is.list(mgweight)) mgweight = unlist(mgweight)
 
-  check.param(param = mgshape, allowvec = TRUE) # do not check positivity in likelihood
-  check.param(param = mgscale, allowvec = TRUE) # do not check positivity in likelihood
-  check.param(param = mgweight, allowvec = TRUE) # do not check positivity in likelihood
+  check.param(mgshape, allowvec = TRUE)
+  check.param(mgscale, allowvec = TRUE)
+  check.param(mgweight, allowvec = TRUE)
 
   # How many components in mixture
-  M = check.inputn(c(length(mgshape), length(mgscale), length(mgweight)))
+  M = check.inputn(c(length(mgshape), length(mgscale), length(mgweight)), allowscalar = TRUE)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -720,15 +724,16 @@ lmgammagpdcon <- function(x, mgshape, mgscale, mgweight, u, xi, phiu = TRUE, log
 # (wrapper for likelihood, inputs and checks designed for optimisation)
 nlmgammagpdcon <- function(pvector, x, M, phiu = TRUE, finitelik = FALSE) {
 
+  check.n(M)
+  if (M == 1) stop("use nlgammagpdcon instead")
   np = 3*M + 2 # maximum number of parameters
 
   # Check properties of inputs
   check.param(pvector, allowvec = TRUE)
   check.nparam(pvector, nparam = np - 1)
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
-  check.n(M)
+  check.logic(finitelik)
 
   mgshape = pvector[1:M]
   mgscale = pvector[(M + 1):(2*M)]
@@ -754,16 +759,17 @@ nlmgammagpdcon <- function(pvector, x, M, phiu = TRUE, finitelik = FALSE) {
 # (wrapper for likelihood, designed for threshold to be fixed and other parameters optimised)
 nlumgammagpdcon <- function(pvector, u, x, M, phiu = TRUE, finitelik = FALSE) {
 
+  check.n(M)
+  if (M == 1) stop("use nlugammagpdcon instead")
   np = 3*M + 2 # maximum number of parameters
 
   # Check properties of inputs
   check.param(pvector, allowvec = TRUE)
   check.nparam(pvector, nparam = np - 2)
   check.posparam(u)
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
-  check.n(M)
+  check.logic(finitelik)
 
   mgshape = pvector[1:M]
   mgscale = pvector[(M + 1):(2*M)]
@@ -788,6 +794,8 @@ nlumgammagpdcon <- function(pvector, u, x, M, phiu = TRUE, finitelik = FALSE) {
 # with component probabilities separated for EM algorithm
 nlEMmgammagpdcon <- function(pvector, tau, mgweight, x, M, phiu = TRUE, finitelik = FALSE) {
 
+  check.n(M)
+  if (M == 1) stop("use nlgammagpdcon instead")
   np.noweight = 2*M + 2 # maximum number of parameters
 
   # Check properties of inputs
@@ -798,10 +806,9 @@ nlEMmgammagpdcon <- function(pvector, tau, mgweight, x, M, phiu = TRUE, finiteli
   
   if (abs(sum(mgweight) - 1) > 1e-6) stop("component weights must sum to one")
 
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
-  check.n(M)
+  check.logic(finitelik)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -875,19 +882,20 @@ nlEMmgammagpdcon <- function(pvector, tau, mgweight, x, M, phiu = TRUE, finiteli
 proflumgammagpdcon <- function(u, pvector, x, M, phiu = TRUE,
   method = "BFGS", control = list(maxit = 10000), finitelik = FALSE, ...) {
 
+  check.n(M)
+  if (M == 1) stop("use proflugammagpdcon instead")
   np = 3*M + 2 # maximum number of parameters
 
   # Check properties of inputs
   check.param(pvector, allowvec = TRUE)
   check.nparam(pvector, nparam = np - 2, allownull = TRUE)
-  check.posparam(param = u)
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.posparam(u)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
-  check.n(M)
+  check.logic(finitelik)
   check.optim(method)
   check.control(control)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
     
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -1057,6 +1065,8 @@ proflumgammagpdcon <- function(u, pvector, x, M, phiu = TRUE,
 # (wrapper for likelihood, designed for threshold to be fixed and other parameters optimised)
 nluEMmgammagpdcon <- function(pvector, u, tau, mgweight, x, M, phiu = TRUE, finitelik = FALSE) {
 
+  check.n(M)
+  if (M == 1) stop("use nlugammagpdcon instead")
   np.noweight = 2*M + 2 # maximum number of parameters
 
   # Check properties of inputs
@@ -1068,10 +1078,9 @@ nluEMmgammagpdcon <- function(pvector, u, tau, mgweight, x, M, phiu = TRUE, fini
   
   if (abs(sum(mgweight) - 1) > 1e-6) stop("component weights must sum to one")
     
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
-  check.n(M)
+  check.logic(finitelik)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
