@@ -146,31 +146,32 @@
 #'   following elements
 #'
 #' \tabular{ll}{
-#'  \code{call}:    \tab \code{optim} call\cr
-#'  \code{x}:       \tab data vector \code{x}\cr
-#'  \code{init}:    \tab \code{pvector}\cr
-#'  \code{fixedu}:  \tab fixed threshold, logical\cr
-#'  \code{useq}:    \tab threshold vector for profile likelihood or scalar for fixed threshold\cr
-#'  \code{optim}:   \tab complete \code{optim} output\cr
-#'  \code{mle}:     \tab vector of MLE of parameters\cr
-#'  \code{cov}:     \tab variance-covariance matrix of MLE of parameters\cr
-#'  \code{se}:      \tab vector of standard errors of MLE of parameters\cr
-#'  \code{rate}:    \tab \code{phiu} to be consistent with \code{\link[evd:fpot]{evd}}\cr
-#'  \code{nllh}:    \tab minimum negative log-likelihood\cr
-#'  \code{n}:       \tab total sample size\cr
-#'  \code{nmean}:   \tab MLE of normal mean\cr
-#'  \code{nsd}:     \tab MLE of normal standard deviation\cr
-#'  \code{u}:       \tab threshold (fixed or MLE)\cr
-#'  \code{sigmau}:  \tab MLE of GPD scale\cr
-#'  \code{xi}:      \tab MLE of GPD shape\cr
-#'  \code{phiu}:    \tab MLE of tail fraction (bulk model or parameterised approach)\cr
-#'  \code{se.phiu}: \tab standard error of MLE of tail fraction\cr
+#'  \code{call}:      \tab \code{optim} call\cr
+#'  \code{x}:         \tab data vector \code{x}\cr
+#'  \code{init}:      \tab \code{pvector}\cr
+#'  \code{fixedu}:    \tab fixed threshold, logical\cr
+#'  \code{useq}:      \tab threshold vector for profile likelihood or scalar for fixed threshold\cr
+#'  \code{nllhuseq}:  \tab profile negative log-likelihood at each threshold in useq\cr
+#'  \code{optim}:     \tab complete \code{optim} output\cr
+#'  \code{mle}:       \tab vector of MLE of parameters\cr
+#'  \code{cov}:       \tab variance-covariance matrix of MLE of parameters\cr
+#'  \code{se}:        \tab vector of standard errors of MLE of parameters\cr
+#'  \code{rate}:      \tab \code{phiu} to be consistent with \code{\link[evd:fpot]{evd}}\cr
+#'  \code{nllh}:      \tab minimum negative log-likelihood\cr
+#'  \code{n}:         \tab total sample size\cr
+#'  \code{nmean}:     \tab MLE of normal mean\cr
+#'  \code{nsd}:       \tab MLE of normal standard deviation\cr
+#'  \code{u}:         \tab threshold (fixed or MLE)\cr
+#'  \code{sigmau}:    \tab MLE of GPD scale\cr
+#'  \code{xi}:        \tab MLE of GPD shape\cr
+#'  \code{phiu}:      \tab MLE of tail fraction (bulk model or parameterised approach)\cr
+#'  \code{se.phiu}:   \tab standard error of MLE of tail fraction\cr
 #' }
 #' 
 #' The output list has some duplicate entries and repeats some of the inputs to both 
 #' provide similar items to those from \code{\link[evd:fpot]{fpot}} and increase usability.
 #'  
-#' @note Unlike all the distribution functions for the extreme value mixture models,
+#' @note Unlike most of the distribution functions for the extreme value mixture models,
 #' the MLE fitting only permits single scalar values for each parameter and 
 #' \code{phiu}.
 #' 
@@ -224,12 +225,15 @@
 #' 
 #' @author Yang Hu and Carl Scarrott \email{carl.scarrott@@canterbury.ac.nz}
 #'
-#' @section Acknowledgments: Apects of these functions are deliberately similar
-#'   in syntax and function with the commonly used functions in the
+#' @section Acknowledgments: These functions are deliberately similar
+#'   in syntax and functionality to the commonly used functions in the
 #'   \code{\link[ismev:gpd.fit]{ismev}} and \code{\link[evd:fpot]{evd}} packages
-#'   for which their author's contributions are gratefully acknowledged. Anna
-#'   MacDonald and Xin Zhao laid some of the groundwork with programs they
-#'   wrote for MATLAB. Clement Lee and Emma Eastoe suggested providing inbuilt
+#'   for which their author's contributions are gratefully acknowledged.
+#'   
+#'   Anna MacDonald and Xin Zhao laid some of the groundwork with programs they
+#'   wrote for MATLAB.
+#'   
+#'   Clement Lee and Emma Eastoe suggested providing inbuilt
 #'   profile likelihood estimation for threshold and fixed threshold approach.
 #' 
 #' @seealso \code{\link[stats:Normal]{dnorm}},
@@ -240,7 +244,9 @@
 #' 
 #' @examples
 #' \dontrun{
-#' par(mfrow=c(2,1))
+#' set.seed(1)
+#' par(mfrow = c(2, 1))
+#' 
 #' x = rnorm(1000)
 #' xx = seq(-4, 4, 0.01)
 #' y = dnorm(xx)
@@ -283,15 +289,17 @@ fnormgpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NULL
 
   call <- match.call()
     
+  np = 5 # maximum number of parameters
+
   # Check properties of inputs
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
-  check.logic(logicarg = phiu) # only logical
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
+  check.logic(phiu)
   check.param(useq, allowvec = TRUE, allownull = TRUE)
-  check.logic(logicarg = fixedu)
-  check.logic(logicarg = std.err)
+  check.logic(fixedu)
+  check.logic(std.err)
   check.optim(method)
   check.control(control)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
@@ -300,7 +308,6 @@ fnormgpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NULL
 
   check.quant(x)
   n = length(x)
-  np = 5 # maximum number of parameters
 
   if ((method == "L-BFGS-B") | (method == "BFGS")) finitelik = TRUE
   
@@ -344,7 +351,7 @@ fnormgpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NULL
       u = useq
     }
 
-    if (fixedu) { # threshold fixed (4 parameters)
+    if (fixedu) { # threshold fixed
       if (is.null(pvector)) {
         pvector[1] = mean(x, trim = 0.2)
         pvector[2] = sd(x)
@@ -352,7 +359,7 @@ fnormgpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NULL
         pvector[3] = initfgpd$sigmau
         pvector[4] = initfgpd$xi
       }
-    } else { # threshold as initial value in usual MLE
+    } else { # threshold free parameter
       if (is.null(pvector)) {
         pvector[1] = mean(x, trim = 0.2)
         pvector[2] = sd(x)
@@ -370,8 +377,10 @@ fnormgpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NULL
 
   if (fixedu) { # fixed threshold (separable) likelihood
     nllh = nlunormgpd(pvector, u, x, phiu)
-    if (is.infinite(nllh)) pvector[4] = 0.1
-    nllh = nlunormgpd(pvector, u, x, phiu)
+    if (is.infinite(nllh)) {
+      pvector[4] = 0.1
+      nllh = nlunormgpd(pvector, u, x, phiu)
+    }
     if (is.infinite(nllh)) stop("initial parameter values are invalid")
   
     fit = optim(par = as.vector(pvector), fn = nlunormgpd, u = u, x = x, phiu = phiu,
@@ -385,8 +394,10 @@ fnormgpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NULL
   } else { # complete (non-separable) likelihood
     
     nllh = nlnormgpd(pvector, x, phiu)
-    if (is.infinite(nllh)) pvector[5] = 0.1
-    nllh = nlnormgpd(pvector, x, phiu)
+    if (is.infinite(nllh)) {
+      pvector[5] = 0.1
+      nllh = nlnormgpd(pvector, x, phiu)
+    }
     if (is.infinite(nllh)) stop("initial parameter values are invalid")
   
     fit = optim(par = as.vector(pvector), fn = nlnormgpd, x = x, phiu = phiu,
@@ -417,14 +428,14 @@ fnormgpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NULL
   if (std.err) {
     qrhess = qr(fit$hessian)
     if (qrhess$rank != ncol(qrhess$qr)) {
-      warning("observed information matrix is singular; use std.err = FALSE")
+      warning("observed information matrix is singular")
       se = NULL
       invhess = NULL
     } else {
       invhess = solve(qrhess)
       vars = diag(invhess)
       if (any(vars <= 0)) {
-        warning("observed information matrix is singular; use std.err = FALSE")
+        warning("observed information matrix is singular")
         invhess = NULL
         se = NULL
       } else {
@@ -436,8 +447,10 @@ fnormgpd <- function(x, phiu = TRUE, useq = NULL, fixedu = FALSE, pvector = NULL
     se = NULL
   }
   
+  if (!exists("nllhu")) nllhu = NULL
+  
   list(call = call, x = as.vector(x), init = as.vector(pvector),
-    fixedu = fixedu, useq = useq, optim = fit, conv = conv, cov = invhess,
+    fixedu = fixedu, useq = useq, nllhuseq = nllhu, optim = fit, conv = conv, cov = invhess,
     mle = fit$par, se = se, rate = phiu, nllh = fit$value, n = n,
     nmean = nmean, nsd = nsd, u = u, sigmau = sigmau, xi = xi, phiu = phiu, se.phiu = se.phiu)
 }
@@ -451,27 +464,30 @@ lnormgpd <- function(x, nmean = 0, nsd = 1, u = qnorm(0.9, nmean, nsd),
   sigmau = nsd, xi = 0, phiu = TRUE, log = TRUE) {
 
   # Check properties of inputs
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
-  check.param(param = nmean)
-  check.param(param = nsd)    # do not check positivity in likelihood
-  check.param(param = u)
-  check.param(param = sigmau) # do not check positivity in likelihood
-  check.param(param = xi)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
+  check.param(nmean)
+  check.param(nsd)
+  check.param(u)
+  check.param(sigmau)
+  check.param(xi)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = log)
+  check.logic(log)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
     x = x[is.finite(x)] # ignore missing and infinite cases
   }
 
-  check.inputn(c(length(nmean), length(nsd), length(u), length(sigmau), length(xi), length(phiu)))
+  check.quant(x)
+  n = length(x)
+
+  check.inputn(c(length(nmean), length(nsd), length(u), length(sigmau), length(xi), length(phiu)),
+               allowscalar = TRUE)
 
   # assume NA or NaN are irrelevant as entire lower tail is now modelled
   # inconsistent with evd library definition
   # hence use which() to ignore these
 
-  n = length(x)
   xu = x[which(x > u)]
   nu = length(xu)
   xb = x[which(x <= u)]
@@ -522,9 +538,9 @@ nlnormgpd <- function(pvector, x, phiu = TRUE, finitelik = FALSE) {
 
   # Check properties of inputs
   check.nparam(pvector, nparam = np)
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
 
   nmean = pvector[1]
   nsd = pvector[2]
@@ -556,15 +572,19 @@ proflunormgpd <- function(u, pvector, x, phiu = TRUE, method = "BFGS",
   # Check properties of inputs
   check.nparam(pvector, nparam = np - 1, allownull = TRUE)
   check.param(u)
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.optim(method)
+  check.control(control)
+  check.logic(finitelik)
 
   if (any(!is.finite(x))) {
     warning("non-finite cases have been removed")
     x = x[is.finite(x)] # ignore missing and infinite cases
   }
 
+  check.quant(x)
+
   # check initial values for other parameters, try usual alternative
   if (!is.null(pvector)) {
     nllh = nlunormgpd(pvector, u, x, phiu)
@@ -581,8 +601,10 @@ proflunormgpd <- function(u, pvector, x, phiu = TRUE, method = "BFGS",
     nllh = nlunormgpd(pvector, u, x, phiu)
   }  
 
-  if (is.infinite(nllh)) pvector[4] = 0.1
+  if (is.infinite(nllh)) {
+    pvector[4] = 0.1
     nllh = nlunormgpd(pvector, u, x, phiu)
+  }
   
   # if still invalid then output cleanly
   if (is.infinite(nllh)) {
@@ -615,9 +637,9 @@ nlunormgpd <- function(pvector, u, x, phiu = TRUE, finitelik = FALSE) {
   # Check properties of inputs
   check.nparam(pvector, nparam = np - 1)
   check.param(u)
-  check.quant(x, allowmiss = TRUE, allowinf = TRUE)
+  check.quant(x, allowna = TRUE, allowinf = TRUE)
   check.phiu(phiu, allowfalse = TRUE)
-  check.logic(logicarg = finitelik)
+  check.logic(finitelik)
     
   nmean = pvector[1]
   nsd = pvector[2]
